/* $Id: channels.c,v 1.14 2002/04/30 11:44:03 leemh Exp $
 * -------------------------------------------------------
 * Copyright (c) 1998-2002 Sebastian Kienzl <zap@riot.org>
 *           (c) 2002 Lee Hardy <lee@leeh.co.uk>
 * -------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "muh.h"
#include "channels.h"
#include "table.h"
#include "log.h"

channellist_type channels;

/* add_channel()
 *
 * adds a channel to muhs internal list
 */
void add_channel(char *s)
{
    struct LogHash *ptr;
    int i;
    int indx = -1;

    /* check to see we're not trying to add a channel we're already in */
    for(i = 0; i < channels.amount; i++)
    {
        if(channels.data[i] && xstrcmp(channels.data[i], s) == 0)
	    return;
    }

    for(i = 0; i < channels.amount; i++)
    {
        if(!channels.data[i])
        {
	    indx = i;
	    break;
	}
    }

    /* xreallocate the struct memory for the channel name */
    if(indx < 0)
    {
        channels.data = xrealloc( (void *)channels.data, ++channels.amount * sizeof( void * ) );
	indx = channels.amount - 1;
    }

    channels.data[indx] = strdup(s);

    /* xreallocate the struct memory for the topic stuff */
    channels.topic = xrealloc((void *)channels.topic, channels.amount * sizeof(void *));
    channels.topicwho = xrealloc((void *)channels.topicwho, channels.amount * sizeof(void *));
    channels.topicwhen = xrealloc((void *)channels.topicwhen, channels.amount * sizeof(void *));

    channels.topic[indx] = NULL;
    channels.topicwho[indx] = NULL;
    channels.topicwhen[indx] = NULL;

    if((ptr = find_log_any(channels.data[indx], 0)))
        open_logfile(ptr);

    else if(global_logtype)
	add_log_global(channels.data[indx]);
}

/* rem_channel()
 *
 * removes a channel from muhs internal list
 */
void rem_channel(char *s)
{
    int i;
    for(i = 0; i < channels.amount; i++)
    {
        if(channels.data[i] && xstrcmp(channels.data[i], s) == 0)
	{
	    if((i >= 0) && (i < channels.amount))
	    {
#ifdef DEBUG
                printf("deleting channel entry %d, compacting table: ", i);
#endif
                FREE(channels.data[i]);
		FREE(channels.topic[i]);
		FREE(channels.topicwho[i]);
		FREE(channels.topicwhen[i]);
            }

            compact_channels_table(&channels.amount);
        }
    }
}

/* compact_channels_table()
 *
 * compacts memory on the channels struct
 */
void compact_channels_table(int *amount)
{
    int i = *amount - 1;

    while(i >= 0)
    {
        if(!channels.data[i])
	{
	    (*amount)--;
	    channels.data = (void *)xrealloc((void *)channels.data, *amount * sizeof(void *));
	    channels.topic = (void *)xrealloc((void *)channels.topic, *amount * sizeof(void *));
	    channels.topicwho = (void *)xrealloc((void *)channels.topicwho, *amount * sizeof(void *));
	    channels.topicwhen = (void *)xrealloc((void *)channels.topicwhen, *amount * sizeof(void *));

	    if(!(*amount))
	    {
	        channels.data = NULL;
		channels.topic = NULL;
		channels.topicwho = NULL;
		channels.topicwhen = NULL;
	    }

	}
	else
	    i = -1;

        i--;
    }
}

/* find_channel()
 *
 * searches for a channel, returning its index or -1 if it doesnt exist
 */
int find_channel(char *p)
{
    int i;

    if(!channels.amount)
	return -1;

    for(i = 0; i < channels.amount; i++)
    {
        if(channels.data[i] && xstrcmp(channels.data[i], p) == 0)
	    return i;
    }

    return -1;
}

/* drop_channels()
 *
 * removes all channels and topics from muhs internal list
 */
void drop_channels()
{
    int i;

    close_logs();
    
    for(i = 0; i < channels.amount; i++)
    {
	FREE(channels.data[i]);
	FREE(channels.topic[i]);
	FREE(channels.topicwho[i]);
	FREE(channels.topicwhen[i]);
    }

    FREE(channels.data);
    FREE(channels.topic);
    FREE(channels.topicwho);
    FREE(channels.topicwhen);

    channels.amount = 0;
}

/* list_channels()
 *
 * returns a comma seperated list of all the channels we're in
 */
char *list_channels()
{
    int i;
    char *temp;

    if(!channels.amount)
	return NULL;

    temp = xmalloc(1);
    temp[0] = 0;

    for(i = 0; i < channels.amount; i++)
    {
        if(channels.data[i])
	{
            temp = xrealloc(temp, strlen(temp) + strlen(channels.data[i]) + 2);
            strcat(temp, channels.data[i]);
            strcat(temp, ",");
        }
    }

    /* remove trailing , */
    temp[strlen(temp) - 1] = 0;
    return temp;
}

/* channel_topic()
 *
 * stores a topic for a channel we're in
 */
void channel_topic(char *channel, char *topic)
{
    int i;
    int hastopic = 1;
    char *p;

    p = topic;
    p++;

    /* check whether were setting, or unsetting a topic */
    if(!p || *p == '\0')
        hastopic = 0;

    /* try to find the channel we're setting a topic for.. */
    for(i = 0; i < channels.amount; i++)
    {
        if(channels.data[i] && xstrcmp(channels.data[i], channel))
	    continue;

	FREE(channels.topic[i]);

        if(hastopic)
	    channels.topic[i] = strdup(topic);

	/* these have changed, blast old values */
	FREE(channels.topicwho[i]);
	FREE(channels.topicwhen[i]);

	return;
    }
}

/* channel_when()
 *
 * stores who set the topic for a channel we're in
 */
void channel_when(char *channel, char *topicwho, char *topicwhen)
{
    int i;

    for(i = 0; i < channels.amount; i++)
    {
	if(channels.data[i] && xstrcmp(channels.data[i], channel))
 	    continue;

	if(channels.topic[i])
	{
	    FREE(channels.topicwho[i]);
	    FREE(channels.topicwhen[i]);

	    channels.topicwho[i] = strdup(topicwho);
	    channels.topicwhen[i] = strdup(topicwhen);
	}

	return;
    }
}

/* hash_channel()
 *
 * creates a hash value based on the first 25 chars of a channel name
 */
unsigned int hash_channel(char *p)
{
    int i = 25;
    unsigned int hash = 0;

    while(*p && --i)
	hash = (hash << 4) - (hash + (unsigned char)tolower(*p++));

    return (hash & (MAX_CHANNELS - 1));
}

